/****************************************************************
Copyright (c) 1998 Sandro Sigala <ssigala@globalnet.it>.
All rights reserved.

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name of the author not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

The author disclaim all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall the author be liable for any
special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
****************************************************************/

#include "config.h"

#include <kapp.h>
#include <kmsgbox.h>
#include <qfile.h>
#include <stdlib.h>
#include <time.h>

#include "kreptoned.h"
#include "startupdialog.h"
#include "newepisodedialog.h"
#include "episodelistdialog.h"
#include "mainwindow.h"

int newEpisode(KApplication& a)
{
	NewEpisodeDialog d;

	QList<Episode> l;
	loadEpisodeList(l, true);
	loadEpisodeList(l, false);
	for (;;) {
		if (!d.exec())
			return -1;
		if (d.episodeName() == "")
			KMsgBox::message(0, "KRepton Editor",
					 "Invalid episode name!");
		else if (d.episodeFilename() == "")
			KMsgBox::message(0, "KRepton Editor",
					 "Invalid episode filename!");
		else {
			bool warn = false;
			for (Episode *e = l.first(); e != 0; e = l.next())
				if (!stricmp(e->name, d.episodeName())) {
					warn = true;
					KMsgBox::message(0, "KRepton Editor",
							 "An user episode already exists with that name!");
					break;
				} else if (!stricmp(e->filename, d.episodeFilename())) {
					warn = true;
					KMsgBox::message(0, "KRepton Editor",
							 "An user episode already exists with that filename!");
					break;
				}
			if (!warn)
				break;
		}
	}

	QString srcpath;
	QString destpath;

	for (Episode *e = l.first(); e != 0; e = l.next())
		if (e->name == d.copyFrom()) {
			if (e->global)
				srcpath = kapp->kde_datadir() + "/krepton/data";
			else
				srcpath = kapp->localkdedir() + "/share/apps/krepton/data";
			srcpath += "/" + e->filename;
			break;
		}

	destpath = kapp->localkdedir() + "/share/apps/krepton/data";
	destpath += "/" + d.episodeFilename();

	QString filename;
	createPath(destpath);
	
	filename = srcpath + "/info";
	QString name;
	int levels;
	QFile fin(filename);
	if (!fin.open(IO_ReadOnly))
		fatal("Cannot open %s", (const char *)filename);
	QTextStream tin(&fin);
	name = tin.readLine();
	tin >> levels;
	fin.close();

	filename = destpath + "/info";
	QFile fout(filename);
	if (!fout.open(IO_WriteOnly))
		fatal("Cannot open %s for writing", (const char *)filename);
	QTextStream tout(&fout);
	tout << d.episodeName() << '\n';
	tout << levels << '\n';
	fout.close();

	copyFile(srcpath + "/sprites.bmp", destpath + "/sprites.bmp");
	for (int i = 0; i < levels; ++i) {
		filename.sprintf("/map%d", i+1);
		copyFile(srcpath + filename, destpath + filename);
	}

	MainWindow w;
	w.loadEpisode(destpath);
	a.setMainWidget(&w);
	w.show();
	return a.exec();
}

int editEpisode(KApplication& a)
{
	EpisodeListDialog d("Edit Episode");
	if (countUserEpisodes() < 1) {
		KMsgBox::message(0, "KRepton Editor",
				 "There are no user episodes available to edit!");
		return -1;
	} else if (!d.exec())
		return -1;

	MainWindow w;

	QList<Episode> l;
	loadEpisodeList(l, false);
	Episode *e;
	for (e = l.first(); e != 0; e = l.next())
		if (d.episodeName() == e->name) {
			QString path = kapp->localkdedir() + "/share/apps/krepton/data/" + e->filename;
			w.loadEpisode(path);
			break;
		}

	a.setMainWidget(&w);
	w.show();
	return a.exec();
}

int removeEpisode(KApplication&)
{
	EpisodeListDialog d("Remove Episode");
	if (countUserEpisodes() < 1) {
		KMsgBox::message(0, "KRepton Editor",
				 "There are no user episodes available to remove!");
	} else if (d.exec() != 0) {
		QString msg = "Are you sure to remove the episode \"";
		msg += d.episodeName() + "\"?";
		if (KMsgBox::yesNo(0, "KRepton Editor", msg) == 1) {
			QList<Episode> l;
			loadEpisodeList(l, false);
			for (Episode *e = l.first(); e != 0; e = l.next())
				if (e->name == d.episodeName()) {
					removeEpisode(kapp->localkdedir() + "/share/apps/krepton/data/" + e->filename);
					break;
				}
		}
	}

	return -1;
}

int main(int argc, char **argv)
{
	KApplication a(argc, argv);

	srand(time(0));

	StartupDialog d;
	for (;;) {
		int retval;
		switch (d.exec()) {
		case StartupDialog::NewEpisode:
			retval = newEpisode(a);
			if (retval != -1)
				return retval;
			break;
		case StartupDialog::EditEpisode:
			retval = editEpisode(a);
			if (retval != -1)
				return retval;
			break;
		case StartupDialog::RemoveEpisode:
			retval = removeEpisode(a);
			if (retval != -1)
				return retval;
			break;
		default:
			return 0;
		}
	}

	return 0;
}
